/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <string.h>
#include <time.h>
#include "ds2_runtime.h"

DS2Runtime::DS2Runtime()
{
	coords = NULL;
	mapping = NULL;
	p1_all = NULL;
	p2_all = NULL;
	pOverlap_all = NULL;
	dis1_all = NULL;
	dis2_all = NULL;
	maxi_all = NULL;
	heads = NULL;
	B = NULL;
	R = NULL;
	NUM_NODES = 0;
}

void DS2Runtime::loadModel(char* model_file_name)
{
	FILE *model_file;
	int NUM;
	int DIM;
	int BLOCKNUM;
	int MAXI;
	int HEAD_NUM;
	int MAX_ROW;
	
	int i,j;
	
	model_file = fopen(model_file_name,"r");
	assert(model_file != NULL);

	fscanf(model_file, "%d%d%d%d%d%d", &NUM, &DIM, &BLOCKNUM, &MAXI, &HEAD_NUM, &MAX_ROW);
	this->NUM_NODES = NUM;

	coords = (float **)malloc(NUM *sizeof(float*));
	for(i=0;i<NUM;i++)
		coords[i] = (float*)malloc(DIM*sizeof(float));
	for(i=0;i<NUM;i++)
		for(j=0;j<DIM;j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			coords[i][j] = v;
		}


	mapping = (int *) malloc(NUM*sizeof(int));
	for(i=0;i<NUM;i++)
	{
		float v;
		fscanf(model_file,"%f",&v);
		mapping[i] = (int)v;
	}
	
	
	maxi_all = (int *)malloc(sizeof(int) * BLOCKNUM);
	for(i=0;i<BLOCKNUM;i++)
	{
		float v;
		fscanf(model_file,"%f",&v);
		maxi_all[i] = (int)v;
	}

	p1_all = (float**)malloc(BLOCKNUM * sizeof(float*));
	p2_all = (float**)malloc(BLOCKNUM * sizeof(float*));
	pOverlap_all = (float**)malloc(BLOCKNUM * sizeof(float*));
	for(i=0;i<BLOCKNUM;i++)
	{
		p1_all[i] = (float*)malloc(maxi_all[i] * sizeof(float));
		p2_all[i] = (float*)malloc(maxi_all[i] * sizeof(float));
		pOverlap_all[i] = (float*)malloc(maxi_all[i] * sizeof(float));
	}
	

	for(i=0;i<BLOCKNUM;i++)
		for(j=0;j<maxi_all[i];j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			p1_all[i][j] = v;
		}
	
	
	for(i=0;i<BLOCKNUM;i++)
		for(j=0;j<maxi_all[i];j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			p2_all[i][j] = v;
		}
		
	
	for(i=0;i<BLOCKNUM;i++)
		for(j=0;j<maxi_all[i];j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			pOverlap_all[i][j] = v;
		}


	heads = (int*)malloc(sizeof(int) * NUM);
	R = (float*)malloc(HEAD_NUM * sizeof(float));
	B = (float*)malloc(HEAD_NUM * sizeof(float));
	for(i=0;i<NUM;i++)
	{
		float v;
		fscanf(model_file,"%f",&v);
		heads[i] = (int)v;
	}
	

	for(i=0;i<HEAD_NUM;i++)
	{
		float v;
		fscanf(model_file,"%f",&v);
		R[i] = v;
	}


	for(i=0;i<HEAD_NUM;i++)
	{
		float v;
		fscanf(model_file,"%f",&v);
		B[i] = v;
	}

	dis1_all = (float**)malloc(MAX_ROW * sizeof(float*));
	dis2_all = (float**)malloc(MAX_ROW * sizeof(float*));
	for(i=0;i<MAX_ROW;i++)
	{
		dis1_all[i] = (float*)malloc(300*sizeof(float));
		dis2_all[i] = (float*)malloc(300*sizeof(float));
	}
	
	
	for(i=0;i<MAX_ROW;i++)
		for(j=0;j<300;j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			dis1_all[i][j] = v;
		}
	

	for(i=0;i<MAX_ROW;i++)
		for(j=0;j<300;j++)
		{
			float v;
			fscanf(model_file,"%f",&v);
			dis2_all[i][j] = v;
		}
	
}


float DS2Runtime::getSyntheticDistance(int id1, int id2)
{
		float result;
		int clusterID1,clusterID2;
		int blockID;
		int tmp;

		if(id1 == id2)
			return 0;
		
		clusterID1 = mapping[id1];
		clusterID2 = mapping[id2];
		
		tmp = (clusterID1 <= clusterID2) ? clusterID1 : clusterID2;
		clusterID2 = (clusterID1>=clusterID2) ? clusterID1 : clusterID2;
		clusterID1 = tmp;
		

		if(clusterID1 == 1 && clusterID2 == 1)
			blockID = 1;
		else if(clusterID1 ==1 && clusterID2 == 2)
			blockID = 2;
		else if(clusterID1 ==1 && clusterID2 == 3)
			blockID = 3;
		else if(clusterID1 ==1 && clusterID2 == 4)
			blockID = 4;
		else if(clusterID1 ==2 && clusterID2 == 2)
			blockID = 5;
		else if(clusterID1 ==2 && clusterID2 == 3)
			blockID = 6;
		else if(clusterID1 ==2 && clusterID2 ==4)
			blockID = 7;
		else if(clusterID1 ==3 && clusterID2 ==3)
			blockID = 8;
		else if(clusterID1 ==3 && clusterID2 ==4)
			blockID = 9;
		else blockID = 10;
		
		blockID = blockID-1;
		
		int i,j,k;
		int sum_maxi_all[10];
		sum_maxi_all[0] = 0;
		for(i=1;i<10;i++)
			sum_maxi_all[i] = sum_maxi_all[i-1]+maxi_all[i-1];
		
		
		float sum;
		sum = 0;
		for(i=0;i<5;i++)
		{
			sum += pow(coords[id1][i]-coords[id2][i],2);
		}
		
		int v;
		float v_f;
		v_f = sqrt(sum);
		v = (int)v_f;
		
		if(v == 0 || v>=maxi_all[blockID])
		      return v_f;
	
		assert(v>0);
		float prob1, prob2;
		prob1 = p1_all[blockID][v];
		prob2 = prob1 + p2_all[blockID][v];

		srand( id1*id2);
		if(v<2000)
		{   
			float r = rand()/(float)RAND_MAX;
			
			if(r >= prob2-pOverlap_all[blockID][v])
			{
				return v_f;
			}
			
			r = rand()/(float)RAND_MAX;
			
			float **dis;
			if(r<prob1/prob2)
				dis = dis1_all;
			else dis = dis2_all;
			
		
			float rr = rand()/(float)RAND_MAX;
			for(k=0;k<300;k++)
				if(dis[sum_maxi_all[blockID]+v][k] > rr)
					break;
				
			if(k>95 && k<=100)
			{
				return v_f;
			}
			int index;
			index = k-1;
			if(index == -1)
				index = 0;
			

			int err;
			if(index<100)
				err = index;
			else if(index >=299)
				err = (int)(-200 + rand()/(float)RAND_MAX *(-300));
			else err = -1 * (index-99);
			
			v_f = v_f - v_f*err/100.0;
			v = (int)v_f;

			//smooth here
			r = rand()/(float)RAND_MAX;
			if(r<0.333)
				v = (v-1)>0?(v-1):v;
			else if(r>0.667)
				v = v+1;

			if(v>2000)
			{
				v_f = 1000 + rand()/(float)RAND_MAX*1000;
				v = (int)v_f;
			}

			//start local distortion
			if(heads[id1] ==0 && heads[id2] == 0)
			{
				return v_f;
			}

			int id1_tmp,id2_tmp;
			id1_tmp = id1;
			id2_tmp = id2;
			if(heads[id1] == 0)
			{
				id1_tmp = id2;
				id2_tmp = id1;
			}
			if(heads[id1]==1 && heads[id2]==1)
			{
				float r = rand()/(float)RAND_MAX;
				if(r>0.5)
				{
					id1_tmp = id1>id2?id1:id2;
					id2_tmp = id1<id2?id1:id2;	
				}
				else
				{
					id1_tmp = id1>id2?id2:id1;
					id2_tmp = id1<id2?id2:id1;
				}
			}
			
			float rad = R[id1_tmp];
			float bi = B[id1_tmp];
			
			if(v_f<=rad && v_f>bi)
				result = bi*0.99;
			else  result = v_f;//return v;
			
		}
		else result = 1000+rand()/(float)RAND_MAX*1000; 
		
		return result+MIN_RTT;

}

void DS2Runtime::generateMatrix(char *output_mat_name, int start_index, int end_index)
{
	FILE *out_mat;
	out_mat = fopen(output_mat_name,"w");
	assert(out_mat!=NULL);	
	
	int id1,id2;
	float result;
	for(id1=start_index; id1<=end_index; id1++)
	{
	   for(id2=start_index; id2<=end_index; id2++)
           {
		result = getSyntheticDistance(id1,id2);

		fprintf(out_mat,"%.1f ",result);
	   }

	   fprintf(out_mat,"\n");
	}
}

/*
int main(int argc, char** argv)
{
	DS2Runtime dr;
	printf("testing\n");
	
	dr.loadModel("DS2Model");
	dr.generateMatrix("mat_ds2_4k",0,3999);

	return 0;
}
*/
