/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

This GNP class is produced based on Professor T. S. Eugene Ng's GNP code.

Copyright (C) 2006 T. S. Eugene Ng, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/

#ifndef GNP_H
#define GNP_H

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "Utility.h"

//extracted from orignial gnp.c
#define PI 3.141592654
#define VERSION 11     /* added dump_computed matrix/combined dump */
#define MAX_ERR 1e20

//extracted from original simplex_downhill.c
#define TINY 1.0e-10
#define NMAX 500000

//extracted from original extract_coords.c
#define MAXN 20000
#define MAXD 20



typedef enum embeddings {
  ND_SPACE,
  CYLINDER,
  SPHERE,
  D1_PLUS_D2_SPACE,
  LAND_MASS,
  LAST_EM
} embeddings_t;


typedef enum model_generators {
  SFILE,
  LAST_MG
} mg_t;



class CGnp
{
private:
	void saveConfig(FILE* f);
	void fitTargetData();
	void fitTestData();
	void fitRemainData();
	unsigned int myrand();
	float solve(void *d, float *solution, int num, 
			float (*fit_func)(float *, int, void*), int mytry);
	void fitModel();
	
	void read_basic_data(char *matrix_file_name, char *combine_file_name);
	
	void choose_mpsi(float *xy);
	float gradient_decent(float *array, int num, void *stuff, 
				float (*fit_func)(float *, int, void*));
	inline float max(float a, float b);
	inline float min(float a, float b);

	void simplex_downhill(float **simplex, float *values, int d, float ftol,
				float (*obj)(float *, int, void *), int *num_eval,
				void *stuff);
public:
	int gnp(int num_probes, int num_targets,char *model_file_name, char *matrix_file_name, char *combined_file_name);
	void gen_conf(char* conf_name, int size_mat, int num_lm, int dim, int num_run,int b_symmetric);
};

float cylindrical_dist(float *a, float *b);
float fit_p1(float *array, int num, void *stuff);
float fit_p2(float *array, int num, void *stuff);
float fit_p2_subset(float *array, int num, void *stuff);
float spherical_dist(float *a, float *b);
float linear_dist(float *a, float *b);
float d1_plus_d2_dist(float *a, float *b);
double dist(int a, int b);
void extract_coords(int dim, int num_files, char **file_names);
#endif
