/******************************************************************************

Delay Space Synthesizer (DS^2) code distribution

Copyright (C) 2006, Bo Zhang, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

The copyright owner can be contacted by e-mail at bozhang@cs.rice.edu

For more information, please visit DS^2 project homepage: http://www.cs.rice.edu/~eugeneng/research/ds2/

********************************************************************************/

#include "Utility.h"

int CUtility::min(int v1, int v2)
{
	return v1<v2?v1:v2;
}

int CUtility::max(int v1, int v2)
{
	return v1>v2?v1:v2;
}
	
float CUtility::min(float v1, float v2)
{
	return v1<v2?v1:v2;
}

float CUtility::max(float v1, float v2)
{
		return v1>v2?v1:v2;
}
    
float CUtility::min_row(FloatVector v_row)
{
	int size = v_row.size();
	assert(size > 0);
	
	float result = INFINITY;
	for(int i=0; i<size; i++)
		if(v_row[i] < result)
			result = v_row[i];
    
	return result;
	
}

float CUtility::max_row(FloatVector v_row)
{
	int size = v_row.size();
	assert(size > 0);

	float result = NEGINFINITY;
	for(int i=0; i<size; i++)
		if(v_row[i] > result)
			result = v_row[i];
	return result;
}

float CUtility::mean_row(FloatVector v_row)
{
	int size = v_row.size();
	assert(size > 0);
	
	float summ = 0;
	for(int i=0; i<size; i++)
		summ += v_row[i];
	
	return summ/size;
}
	
	
int CUtility::max_row(IntVector v_row)
{
	int size = v_row.size();
	assert(size > 0);
	
	int result = (int)NEGINFINITY;
	for(int i=0; i<size; i++)
		if(v_row[i] > result)
			result = v_row[i];
	
	return result;
}

int CUtility::max_index_row(IntVector v_row)
{
	int size = v_row.size();
	assert(size > 0);
	int index = -1;
	int result = (int)NEGINFINITY;
	
	for(int i=0; i<size; i++)
		if(v_row[i] > result)
		{
			result = v_row[i];
			index = i;
		}
	
	return index;
}

float CUtility::max_row(float *v_row, int size)
{
	float maxi = NEGINFINITY;
	int index = -1;
	for(int i=0; i<size; i++)
		if(v_row[i] > maxi)
		{
			maxi = v_row[i];
		}
	
	return maxi;
}


float CUtility::max_matrix(float **v_mat, int size1, int size2)
{
	float maxi = NEGINFINITY;
	int i;
	for(i=0; i<size1; i++)
	{
		
		float tmp = CUtility::max_row(v_mat[i],size2);
		if(maxi < tmp)
			maxi = tmp;
	}
	return maxi;
}

//find the smallest positive number in the matrix
float CUtility::min_matrix(float **v_mat, int size1, int size2, int &index_i, int &index_j)
{
	int i,j;
	float mini = INFINITY;
	for(i=0; i<size1; i++)
	   for(j=0; j<size2; j++)
	   if(v_mat[i][j]>0 && v_mat[i][j]<mini)
	   {
		index_i = i;
		index_j = j;
		mini = v_mat[i][j];
	   }
	return mini;
}

int CUtility::sum_row(IntVector v_row)
{
	int result = 0;
	for(int i=0; i<v_row.size(); i++)
		result += v_row[i];
	return result;
}

float CUtility::sum_row(FloatVector v_row)
{
	float result = 0;
	for(int i=0; i<v_row.size(); i++)
		result += v_row[i];
	return result;
}

float CUtility::myMin_row(FloatVector v_row, float mini, float maxi)
{
	int size = v_row.size();
	assert(size > 0);
	
	float result = INFINITY;
	
	for(int i=0; i<size; i++)
		if(v_row[i]>mini && v_row[i]<maxi && v_row[i] < result)
			result = v_row[i];
    
	return result;
}

float CUtility::myMax_row(FloatVector v_row, float mini, float maxi)
{
	int size = v_row.size();
	assert(size > 0);

	float result = NEGINFINITY;
	
	for(int i=0; i<size; i++)
		if(v_row[i]>mini && v_row[i]<maxi && v_row[i] > result)
			result = v_row[i];
			
	return result;
}

float CUtility::myMean_row(FloatVector v_row, float mini, float maxi)
{
	int size = v_row.size();
	assert(size > 0);
	
	float summ = 0;
	int count = 0;
	
	for(int i=0; i<size; i++)
		if(v_row[i]>mini && v_row[i]<maxi)
		{
			count++;
			summ += v_row[i];
		}

	assert(count > 0);
	
	return summ/count;
}


float CUtility::abs(float v)
{
	return v>0?v:-v;
}


//from 0 to range-1
IntVector CUtility::randVector(int range, int size, int sortOrNot)
{
	assert(size<=range && range>0 && size>0);
	IntVector vec;
	vec.resize(size);
	int count = 0;
	while(count<size)
	{
		bool b = true;
		int v = CUtility::min(range-1, (int)(floor(range*(rand()/(float)RAND_MAX))));
		for(int i=0; i<count; i++)
			if(vec[i] == v)
			{
				b = false;
				break;
			}
		
		if(b)
			vec[count++] = v;
	}


	if(sortOrNot == 1)
	{
		int i,j;
		for(i=0;i<size;i++)
		{
			int k=i;
			for(j=i+1;j<size;j++)
			{
				if(vec[k] > vec[j])
					k=j;
			}
			
			if(k!=i)
			{
				int tmp;
				tmp = vec[k];
				vec[k] = vec[i];
				vec[i] = tmp;
			}
		}
	}
	return vec;
}


FILE* CUtility::FOPEN(char *fileName, char *mode)
{
	FILE *result;
	result  = fopen(fileName,mode);
	assert(result != NULL);
	return result;
}


float **CUtility::mallocMatrix(int len, int wid)
{
	float **mat;
	mat = (float**)malloc(len * sizeof(float*));
	assert(mat!=NULL);
	
	for(int i=0; i<len; i++)
	{
		mat[i] = (float*)malloc(wid * sizeof(float));
		assert(mat[i] != NULL);
		memset(mat[i], 0, sizeof(float)*wid);
	}
	return mat;
}


void CUtility::freeMatrix(float **mat, int len)
{
	assert(mat != NULL);
	for(int i=0; i<len; i++)
	{
		assert(mat[i] != NULL);
		free(mat[i]);
	}
	free(mat);
}


float** CUtility::readMatrix(char *matName, int len, int wid)
{
	FILE* matFile;
	matFile = CUtility::FOPEN(matName,"r");
	
	float **mat;
	mat = CUtility::mallocMatrix(len,wid);
	for(int i=0; i<len; i++)
		for(int j=0; j<wid; j++)
		{
			float v;
			assert(fscanf(matFile, "%f", &v)==1);
			mat[i][j] = v;
		}
	return mat;
}


void CUtility::writeMatrix(char *matName, float **mat, int len, int wid)
{
	FILE* matFile;
	matFile = CUtility::FOPEN(matName,"w");
	
	for(int i=0; i<len; i++)
	{
		for(int j=0; j<wid; j++)
		{
			fprintf(matFile,"%f ", mat[i][j]);
		}
		fprintf(matFile, "\n");
		fflush(matFile);
	}
	fclose(matFile);
}

void  CUtility::writeMatrix(FILE *file, float **mat, int len, int wid)
{
	for(int i=0; i<len; i++)
	{
		for(int j=0; j<wid; j++)
		{
			fprintf(file,"%f ", mat[i][j]);
		}
		fprintf(file, "\n");
		fflush(file);
	}
}

void CUtility::writeIntVector(char *name, IntVector iv)
{
	FILE *iv_file = CUtility::FOPEN(name, "w");
	for(int i=0; i<iv.size(); i++)
		fprintf(iv_file, "%d ", iv[i]);
	fprintf(iv_file,"\n");
	fflush(iv_file);
	fclose(iv_file);
}

void CUtility::writeFloatVector(char *name, FloatVector fv)
{
	FILE *fv_file = CUtility::FOPEN(name, "w");
	for(int i=0; i<fv.size(); i++)
		fprintf(fv_file, "%f ", fv[i]);
	fprintf(fv_file,"\n");
	fflush(fv_file);
	fclose(fv_file);
}

IntVector CUtility::loadIntVector(char *name)
{
	FILE *iv_file = CUtility::FOPEN(name, "r");
	IntVector iv;
	int tmp;
	while(fscanf(iv_file, "%d", &tmp)==1)
	{
		iv.push_back(tmp);
	}
	return iv;
}

float** CUtility::readCoords(char *coordsName, int dim, int &size)
{
	FILE *coordsFile;
	coordsFile = CUtility::FOPEN(coordsName, "r");
	
	size = 0;
	char buf[300];
	while(fgets(buf, 300, coordsFile) != NULL)
		size++;
	fclose(coordsFile);

	float **res;
	res = CUtility::readMatrix(coordsName, size, dim);
	
	return res;
}


float** CUtility::concat_coords(float** coords1, int size1, float** coords2, int size2, int dim)
{
	float **res;
	res = CUtility::mallocMatrix(size1+size2, dim);
	
	int i,j;
	for(i=0; i<size1; i++)
		for(j=0; j<dim; j++)
			res[i][j] = coords1[i][j];
	
	for(i=size1; i<size1+size2; i++)
		for(j=0; j<dim; j++)
			res[i][j] = coords2[i-size1][j];
	
	freeMatrix(coords1, size1);
	freeMatrix(coords2, size2);
	
	return res;
}

float** CUtility::sample_coords(float** coords, int size, IntVector iv, int dim)
{
	float **res;
	res = mallocMatrix(iv.size(), dim);
	
	int i, j;
	for(i=0; i<iv.size(); i++)
	{
		assert(iv[i]<size);
		for(j=0; j<dim; j++)
		{
			res[i][j] = coords[iv[i]][j];
		}
	}
	return res;
}

void CUtility::sort2(FloatVector fv, IntVector iv, FloatVector *fv_d, IntVector *iv_d)//destination
{
	int i,j;
	int len1 = fv.size();
	int len2 = iv.size();
	assert(len1 == len2 && len1>0);
	
	for(i=0; i<len1-1; i++)
	{
		int k=i;
		for(j=i+1; j<len1; j++)
		{
			if(fv[k] > fv[j])
				k=j;
		}
		if(k != i)
		{
			float tmp;
			tmp = fv[i];
			fv[i] = fv[k];
			fv[k] = tmp;
			
			int tmp2;
			tmp2 = iv[i];
			iv[i] = iv[k];
			iv[k] = tmp2;
		}
	}
	*fv_d = fv;
	*iv_d = iv;
}


FloatVector CUtility::sort(FloatVector fv)
{
	for(int i=0; i<fv.size(); i++)
	{
		int k = i;
		for(int j=i+1; j<fv.size(); j++)
		{
			if(fv[j] < fv[k])
				k = j;
		}
		
		if(k != i)
		{
			float tmp;
			tmp = fv[i];
			fv[i] = fv[k];
			fv[k] = tmp;
		}
	}
	
	return fv;
}

IntVector CUtility::intersect(IntVector v1, IntVector v2)
{
	IntVector::iterator pos1, pos2;
	IntVector res;
	
	for(pos1= v1.begin(); pos1!=v1.end(); pos1++)
		for(pos2=v2.begin(); pos2!=v2.end(); pos2++)
		{
			if(*pos1 == *pos2)
			{
				res.push_back(*pos1);
				break;
			}
		}
	return res;
}

IntVector CUtility::setDiff(IntVector v1, IntVector v2)
{
	IntVector::iterator pos1, pos2;
	IntVector res;
	
	for(pos1= v1.begin(); pos1!=v1.end(); pos1++)
	{
		int inSet = 0;
		for(pos2=v2.begin(); pos2!=v2.end(); pos2++)
		{
			if(*pos1 == *pos2)
			{
				inSet = 1;
				break;
			}
		}
		if(inSet == 0)
			res.push_back(*pos1);
	}
	return res;
}


IntVector CUtility::concat_iv(IntVector v1, IntVector v2)
{
	IntVector::iterator pos;
	IntVector res = v1;
	for(pos=v2.begin(); pos!=v2.end(); pos++)
		res.push_back(*pos);
	return res;
}
